from collections import defaultdict
from xml.etree import cElementTree as ET

# XML to Dict parser, from:
# https://stackoverflow.com/questions/2148119/how-to-convert-an-xml-string-to-a-dictionary-in-python/10077069
# Used in SiEPIC-Tools (https://github.com/lukasc-ubc/SiEPIC-Tools)
def etree_to_dict(t):
    d = {t.tag: {} if t.attrib else None}
    children = list(t)
    if children:
        dd = defaultdict(list)
        for dc in map(etree_to_dict, children):
            for k, v in dc.items():
                dd[k].append(v)
        d = {t.tag: {k: v[0] if len(v) == 1 else v for k, v in dd.items()}}
    if t.attrib:
        d[t.tag].update(('@' + k, v) for k, v in t.attrib.items())
    if t.text:
        text = t.text.strip()
        if children or t.attrib:
            if text:
                d[t.tag]['#text'] = text
        else:
            d[t.tag] = text
    return d


def xml_to_dict(t):
    try:
        e = ET.XML(t)
    except:
        raise UserWarning("Error in the XML file.")
    return etree_to_dict(e)


import os
from sys import platform


def isGSI():
    ''' Detect whether we are in klayout's generic scripting interface
        Since pya cannot be imported from outside, try that.
    '''
    try:
        import pya
        return True
    except ImportError:
        return False


def isGUI():
    ''' Klayout can run as a window or in batch mode on command line.
        If it launches in batch (-b) or database-only (-zz) mode, then main_window is None.

        If however it runs in non-GUI mode (-z), it is not None; however,
        it has not been given a title yet. (see layApplication.cc)
    '''
    if isGSI():
        import pya
        main = pya.Application.instance().main_window()
        if main is not None:
            return True
    return False


def klayout_home():
    ''' Figure out the klayout configuration directory.
        Uses exactly the same logic as used in the klayout source.
    '''
    ly_home = os.environ.get('KLAYOUT_HOME', '')
    if ly_home == '':
        ly_home = os.path.join(os.path.expanduser('~'), 'KLayout' if is_windows() else '.klayout')
    if not os.path.exists(ly_home):
        raise FileNotFoundError('The KLayout config directory was not found. KLayout might not be installed.')
    return ly_home


def is_windows():
    if platform == "linux" or platform == "linux2":
        return False
    elif platform == "darwin":
        return False
    elif platform == "win32":
        return True
    else:
        raise ValueError('Unrecognized operating system: {}'.format(platform))


# determine what we will use for pya
if isGSI():
    import pya
else:
    try:
        import klayout.db as pya
    except ImportError as err:
        pya = None



__version__ = '0.0.12'

from lygadgets.environment import pya, isGUI, isGSI
from lygadgets.messaging import message, message_loud
from lygadgets.markup import xml_to_dict
from lygadgets.system_linker import export_to_system
from lygadgets.salt_linker import postinstall_hook